from django.core.management.base import AppCommand, CommandError
from django.db.models import get_models
from django.template import loader as TemplateLoader, Context
from optparse import make_option
from bricklayer.utils import ColoredOutputMixin
import os
import re
import imp

class Command(AppCommand,ColoredOutputMixin):
    help = """Creates and populates a basic admin.py file if it doesn't exists. For each model in the app, 
            create an AdminModel class (if it doesn't exists) and then register the model on the admin site."""

    args = '[appname ...]'

    option_list = AppCommand.option_list + (
        make_option('--models', '-m', 
                    dest='models', 
                    type="string",
                    default=None,
                    help='Model/s name. Multiple models can be passed separated by a comma. E.g. -m User,Post,Comment'), 
    )

    def handle_app(self, app, **options):
        app_label = app.__name__.split('.')[-2]
        app_path = os.path.dirname(app.__file__)
        admin_filename = os.path.join(os.path.dirname(app.__file__), 'admin.py')
        is_verbose = options['verbosity'] >= 1
        
        # Get the name of each model in the app
        app_models = [model._meta.object_name for model in get_models(app)]
        # if models    
        template = TemplateLoader.get_template("admin.py.template")
        if options['models']:
            # Accept list of models as CSV
            requested_models = options["models"].split(",")
            app_models = filter(lambda model: model in requested_models, app_models)
        
        if not os.path.exists(admin_filename):
            self.stdout.write(app_label+".admin.py created")
            mode = "w"
            context = Context({"app": app_label, 
                               "import_admin": True,
                                "models": app_models,
                                "models_to_define": app_models,
                                "models_to_register": app_models,
                                "models_to_import":app_models})
            
        else:
            self.stdout.write("admin.py already exists")
            """
            Check if each model is already registered.
            For each model which hasn't been registered yet,
            create an AdminClass and register the model.
            """
            admin_module = imp.load_source("admins",os.path.join(app_path, 'admin.py'))
            with open(admin_filename, 'r') as admin_file:
                admin_file_lines = admin_file.readlines()
                admin_file_text = "".join( admin_file_lines  )
            
            mode = 'a'
            models_to_define = []
            models_to_register = []
            models_to_import = []
            for model in app_models:
                regex = re.compile("(admin.site.register\(\s*"+model+"\s*,)",re.MULTILINE)
                
                #if them model wasn't loaded yet...
                if model not in dir(admin_module):
                    models_to_import.append(model)
                    
                #if model has not been registered yet..
                if not regex.search(admin_file_text):
                    models_to_register.append(model)
                
                # If admin model name doesn't collide with an existing one
                if not "%sAdmin"%model in admin_file_text:
                    models_to_define.append(model)
            
            context = Context({"app": app_label,
                               "import_admin": "admin" in dir(admin_module),
                               "models": app_models,
                               "models_to_define": models_to_define,
                               "models_to_import":models_to_import,
                               "models_to_register": models_to_register})

        # Write admin.py   
        with open(admin_filename, mode) as admin_file:
                admin_file.write(template.render(context))
        
        if is_verbose:
            if mode == "w":
                msg = "%s.admins created"
            else:
                msg = "%s.admins updated"
            self.print_success(msg%app_label)
            


from django.core.management.base import BaseCommand, CommandError
from django.core.management import get_commands
from django.utils.importlib import import_module
from django.db import models
from django.core.exceptions import ImproperlyConfigured
from django.template import loader as TemplateLoader, Context
from optparse import make_option
from bricklayer.utils import ColoredOutputMixin, underscore_to_camelcase
import os

class Command(BaseCommand, ColoredOutputMixin):
    option_list = BaseCommand.option_list + (
        make_option('--app', '-a', 
                    dest='appcommand', 
                    action="store_true",
                    default=False,
                    help='Create an AppCommand'),
    )
    help = 'It generates new command skeleton.'
    args = '[command_name, app_label]'
    def handle(self, command_name, app_label, **options):
        
        
        
        assert app_label is not None
        assert command_name is not None
        
        # Check if command with the same name exist
        # to avoid name clashes
        
        if command_name in get_commands().keys():
            raise CommandError("A command with the name '%s' is already defined in %s"%(command_name, get_commands()[command_name]))
        try:      
            app = import_module(app_label)
            # Check if the app is installed
            try:
                models.get_app(app_label)
            except ImproperlyConfigured:
                self.print_warning("WARNING: %s is not installed.")
                
        except ImportError:
            raise Exception("app %s doesn't exist"%app_label)
        
        app_path = os.path.dirname(app.__file__)
        
        management_path = os.path.join(app_path, "management")
        commands_path   = os.path.join(management_path, "commands")
        
        # Create management and management/commands folders
        if not ( os.path.exists(commands_path) and os.path.isdir(commands_path) ):
            os.makedirs(commands_path)
        
        # Add __init__.py file to management folder in order to turn it into
        # a python package.
        management_init_file = os.path.join(management_path,"__init__.py")
        if not os.path.exists(management_init_file):
            with open(management_init_file,"w") as f: f.close()
            self.print_message("management package created")
            
        
        
        # Add __init__.py file to commands folder in order to turn it into
        # a python package.
        # Create a command shortcut.
        commands_init_file = os.path.join(commands_path,"__init__.py")
        template = TemplateLoader.get_template("command.init.py.template")
        context = Context({'app': app_label, 
                           'command_name':command_name,
                           'command_name_camel':underscore_to_camelcase(command_name)})
        if not os.path.exists(commands_init_file):
            mode = 'w'
            self.print_message("commands package created")
        else:
            mode = 'a'
        with open(commands_init_file,mode) as f: 
            f.write(template.render(context))
        
        command_file = os.path.join(commands_path,command_name+".py")
        
        template = TemplateLoader.get_template('appcommand.py.template' if options['appcommand'] else 'command.py.template')
        context = Context({})
        if not os.path.exists(command_file):
            with open(command_file,"w") as f: 
                f.write(template.render(context))
            
            self.print_success("command %s created."%command_name)
        else:
            self.print_error("file %s.py already exists"%command_name) 
           
    
        


from django.conf import settings
from django.core.management.base import AppCommand
from django.db.models import get_models, get_model
from django.template import loader as TemplateLoader, Context
from django.utils.module_loading import import_module
from bricklayer.utils import exists_dir, exists_path, is_blank
from optparse import make_option
import os, imp
import inspect



class Command(AppCommand):
    help = """Creates generic views, urls.py and templates for your models."""
    option_list = AppCommand.option_list + (
        make_option('--models', '-m', 
                    dest='models', 
                    type="string",
                    default=None,
                    help='Model/s name. Multiple models can be passed separated by a comma. E.g. -m User,Post,Comment'),
        make_option('--form', '-f', 
                    dest='forms',
                    action="store_true",
                    default=False,
                    help='Generate a ModelForm for each model'),
        make_option('-c', 
                    dest='create',
                    action="store_true",
                    default=False,
                    help='Prevent create view from being generated'),
        make_option('-r', 
                    dest='read',
                    action="store_true",
                    default=False,
                    help='Prevent list and detial view from being generated'),
        make_option('-u', 
                    dest='update',
                    action="store_true",
                    default=False,
                    help='Prevent update view from being generated'),
        make_option('-d', 
                    dest='delete',
                    action="store_true",
                    default=False,
                    help='Prevent delete view from being generated'),
                                            
                                        
    )
    

    def handle_app(self, app, **options):
                
        # Check if multiple model are passed as arguments
        if options['models'] and ',' in options['models']:
            options['models'] = options['models'].split(',')
        else:
            options['models'] = [ options['models'] ]
        
        is_verbose = options['verbosity'] >= 1
        # some useful shortcuts
        app_models = [model._meta.object_name for model in get_models(app)]
        models = options['models'] or app_models
        app_name = app.__name__.split('.')[-2]
        app_path = os.path.dirname(app.__file__)
        template_path = os.path.join(app_path, 'templates')
        app_template_path = os.path.join(app_path, 'templates', app_name)
       
        # Test if models exists in the app
        if options['models']:
            models = [ model for model in options['models'] if model in app_models ]
        
        # Set which views are going to be generated
        generic_views = ["CreateView", "DetailView", "ListView", "UpdateView", "DeleteView"]
        if options['create']:
            generic_views.remove("CreateView")
        
        if options['read']:
            generic_views.remove("DetailView")
            generic_views.remove("ListView")
        
        if options['update']:
            generic_views.remove("UpdateView")
            
        if options['delete']:
            generic_views.remove("DeleteView")
                
        # Create templates folder if not exists
        if not exists_dir(template_path):
            os.mkdir(template_path)
        
        if not exists_dir(app_template_path):
            os.mkdir(app_template_path)
        
        # Create templates 
        for model in models:
            model_name = model.lower()
            model_class = get_model(app_name, model)
            assert model_class is not None, "model_class is None"
            
            
            
            # Model form template
            if not exists_path(app_template_path, model_name + "_form.html"):
                template = TemplateLoader.get_template('form.html.template')
                context = Context({"model":model})
                with open(os.path.join(app_template_path, model_name + "_form.html" ),'w') as f:
                    f.write(template.render(context))
            
            # Model confirm delete template
            if not exists_path(app_template_path, model_name + "_confirm_delete.html") and "DeleteView" in generic_views:
                template = TemplateLoader.get_template('confirm_delete.html.template')
                context = Context({"model":model})
                with open(os.path.join(app_template_path, model_name + "_confirm_delete.html" ),'w') as f:
                    f.write(template.render(context))
            
            # Model list template
            
            if not exists_path(app_template_path, model_name + "_list.html") and "ListView" in generic_views:
                template = TemplateLoader.get_template('list.html.template')
                context = Context({'model':model, 
                                   'verbose_name': model_class._meta.verbose_name,
                                   'verbose_name_plural':model_class._meta.verbose_name_plural })
                with open(os.path.join(app_template_path, model_name + "_list.html" ),'w') as f:
                    f.write(template.render(context))
            
            
            # Model detail template
            if not exists_path(app_template_path, model_name + "_detail.html") and "DetailView" in generic_views:
                template = TemplateLoader.get_template('detail.html.template')
                context = Context({'model':model, 
                                   'fields': [ field.name for field in model_class._meta.fields] })
                with open(os.path.join(app_template_path, model_name + "_detail.html" ),'w') as f:
                    f.write(template.render(context))
            
            # Create forms.py
            if options["forms"]:
                from bricklayer.management.commands import GenerateFormCommand
                import copy
                command = GenerateFormCommand()
                form_options = copy.copy(options)
                form_options["models"] = model
                command.handle_app(app, **form_options)
                
            # Create views.py
            # If views.py exists (it should exists), it generates a view
            # only if one with that name doesn't already exist.
            template = TemplateLoader.get_template("views.py.template")
            
            if not exists_path(app_path, 'views.py') or is_blank(app_path, "views.py"):
                mode = "w"
                context  = Context({'app': app_name,
                                    'model': model,
                                    'has_form': options['forms'],
                                    'generic_views': generic_views,
                                    'import_model': True,
                                    'import_reverse_lazy': True,
                                    'import_form': options['forms'],
                                    'create': "CreateView" in generic_views,
                                    'detail': "DetailView" in generic_views,
                                    'list':   "ListView"   in generic_views,
                                    'update': "UpdateView" in generic_views,
                                    'delete': "DeleteView" in generic_views})
            else:
                views_module = dir( imp.load_source("views",os.path.join(app_path, 'views.py')) )
                mode ="a"
                generic_views = filter(lambda view: view not in views_module, generic_views)
                context  = Context({'app': app_name,
                                    'model': model,
                                    'import_model': model not in views_module,
                                    'import_reverse_lazy': "reverse_lazy" not in views_module ,
                                    'import_form': options['forms'] and model+"Form" not in views_module ,
                                    'has_form': options['forms'],
                                    'generic_views': generic_views,
                                    'create': model+"Create" not in views_module and "CreateView" in generic_views,
                                    'detail': model+"Detail" not in views_module and "DetailView" in generic_views,
                                    'list':   model+"List"   not in views_module and "ListView"   in generic_views,
                                    'update': model+"Update" not in views_module and "UpdateView" in generic_views,
                                    'delete': model+"Delete" not in views_module and "DeleteView" in generic_views})
                
            with open(os.path.join(app_path, 'views.py'),mode) as f:
                    f.write(template.render(context))
            
                
            # Create urls.py
            template = TemplateLoader.get_template("urls.py.template")
            imports = ['url', 'patterns']
            generic_view_types = map(lambda v: v.replace("View",""), generic_views) 
            if not exists_path(app_path, 'urls.py'):
                context  = Context({'app': app_name,
                                    'model': model,
                                    'imports':imports,
                                    'extend_urlpatterns': False,
                                    'generic_view_types':generic_view_types })
                
                mode = "w"
            else:
                urls_module = dir( imp.load_source("urls",os.path.join(app_path, 'urls.py')) )
                imports = filter( lambda func: func not in urls_module, imports)
                extend_urlpatterns = 'urlpatterns' in urls_module
                    
                context  = Context({'app': app_name,
                                    'model_name':model_name, 
                                    'model': model,
                                    'imports':imports,
                                    'extend_urlpatterns': extend_urlpatterns,
                                    'generic_view_types': generic_view_types })
                mode = "a"
            
            with open(os.path.join(app_path, 'urls.py'),mode) as f:
                    f.write(template.render(context))
            
            
            root_urlconf_path = inspect.getsourcefile(import_module(settings.ROOT_URLCONF)) 
            with open(root_urlconf_path, "r") as f:
                urls_text = "".join(f.readlines())
            with open(root_urlconf_path, "a") as f:
                extend_patterns = "\nurlpatterns+=patterns(\"\",url(r\"%s/\",include(\"%s.urls\")),)"%(model_name, app_name)
                if extend_patterns not in urls_text:
                    f.write(extend_patterns)
        
            
            


from django.conf import settings
from django.test import TestCase
from django.utils.module_loading import import_module
from bricklayer.utils import run_manage_py, exists_path, exists_dir
import django, inspect, os, compileall, imp


        
class GenerateCRUDTestCase(TestCase):
    def setUp(self):
        run_manage_py("generatecrud test_app -m TestModel")
        import test_app
        self.app_path = os.path.abspath(test_app.__path__[0])
        compileall.compile_dir(self.app_path, force=1)
    
    def test_views(self):
        from test_app import views as views_py
        
        msg = "views.py contains " + ",".join(dir(views_py))
        with open(os.path.join(self.app_path, "views.py")) as f:
            msg += "".join(f.readlines())
        self.assertTrue(hasattr(views_py, "TestModelCreate"), msg)
    
        
    def test_crud_files(self):
        app_path = self.app_path
        self.assertTrue(exists_path(app_path,'urls.py'))
        self.assertTrue(exists_path(app_path,'templates'))
        self.assertTrue(exists_dir( app_path,'templates'))
        self.assertTrue(exists_dir( app_path,'templates', 'test_app'))
        self.assertTrue(exists_path(app_path,'templates','test_app','testmodel_form.html'))
        self.assertTrue(exists_path(app_path,'templates','test_app','testmodel_list.html'))
        self.assertTrue(exists_path(app_path,'templates','test_app','testmodel_detail.html'))
        self.assertTrue(exists_path(app_path,'templates','test_app','testmodel_confirm_delete.html'))
        
    def tearDown(self):
        app_path = self.app_path
        os.remove(os.path.join(app_path,'templates','test_app','testmodel_form.html'))
        os.remove(os.path.join(app_path,'templates','test_app','testmodel_list.html'))
        os.remove(os.path.join(app_path,'templates','test_app','testmodel_detail.html'))
        os.remove(os.path.join(app_path,'templates','test_app','testmodel_confirm_delete.html'))
        os.rmdir(os.path.join(app_path, 'templates','test_app'))
        os.rmdir(os.path.join(app_path, 'templates'))
        os.remove(os.path.join(app_path, 'urls.py'))
        try:
            os.remove(os.path.join(app_path, 'forms.py'))
        except: pass
        
        os.remove(os.path.join(app_path, 'views.py'))
        with open(os.path.join(app_path, 'views.py'),"w") as f:
                f.write("")
        
        root_urlconf_path = inspect.getsourcefile(import_module(settings.ROOT_URLCONF)) 
        with open(root_urlconf_path, "w+") as f:
            urls_text = f.read()
            extend_patterns = 'urlpatterns+=patterns("",url(r"testmodel/",include("test_app.urls")),)'
            urls_text = urls_text.replace(extend_patterns, "\n")
            f.write(urls_text)
            
class GenerateCRUDandFormTestCase(TestCase):
    def setUp(self):
        import test_app
        self.app_path = os.path.abspath(test_app.__path__[0])
        with open(os.path.join(self.app_path, 'views.py'),'w+') as f:
            views = f.read()
            views = views.strip()
            if views != '':
                f.write("")
        run_manage_py("generatecrud test_app -m TestModel -f")
        compileall.compile_dir(self.app_path, force=1)
        
        
     
    def test_form_py(self):
        self.assertTrue(exists_path(self.app_path, 'forms.py'))
        forms_module = dir(imp.load_source("forms", os.path.join(self.app_path, "forms.py")))
        self.assertIn("forms", forms_module)
        self.assertIn("TestModelForm", forms_module)
     
    def test_view_py(self):
        app_path = self.app_path
        forms = imp.load_source("forms",os.path.join(app_path, 'forms.py')) 
        views = imp.load_source("views",os.path.join(app_path, 'views.py'))
        self.assertTrue(hasattr(views, "TestModelCreate"))
        self.assertTrue(hasattr(views, "TestModelUpdate"))
        self.assertTrue(hasattr(forms, "TestModelForm"))
     
    def tearDown(self):
        app_path = self.app_path
        os.remove(os.path.join(app_path,'templates','test_app','testmodel_form.html'))
        os.remove(os.path.join(app_path,'templates','test_app','testmodel_list.html'))
        os.remove(os.path.join(app_path,'templates','test_app','testmodel_detail.html'))
        os.remove(os.path.join(app_path,'templates','test_app','testmodel_confirm_delete.html'))
        os.rmdir(os.path.join(app_path, 'templates','test_app'))
        os.rmdir(os.path.join(app_path, 'templates'))
        os.remove(os.path.join(app_path, 'urls.py'))
        try:
            os.remove(os.path.join(app_path, 'forms.py'))
        except: pass
        
        os.remove(os.path.join(app_path, 'views.py'))
        with open(os.path.join(app_path, 'views.py'),"w") as f:
                f.write("")
        
        root_urlconf_path = inspect.getsourcefile(import_module(settings.ROOT_URLCONF)) 
        with open(root_urlconf_path, "w+") as f:
            urls_text = f.read()
            extend_patterns = 'urlpatterns+=patterns("",url(r"testmodel/",include("test_app.urls")),)'
            urls_text = urls_text.replace(extend_patterns, "\n")
            f.write(urls_text)


from django.core.management import get_commands, CommandError,load_command_class,BaseCommand, execute_from_command_line
from django.core.management.base import BaseCommand
from django.utils import termcolors
from optparse import OptionParser
from sys import stdout, stderr
from os.path import exists, join, isdir


SUCCESS_MESSAGE = termcolors.make_style(fg='green', opts=('bold',))

def underscore_to_camelcase(word, lower_first=False):
    result = ''.join(char.capitalize() for char in word.split('_'))
    if lower_first:
        return result[0].lower() + result[1:]
    else:
        return result

def run_manage_py(command):
    """
        Run console-like command.
        e.g.
        " run_manage_py("syncdb") "
        is equivalent to
        " $ python manage.py syncdb "
        
    """
    args = map(lambda token: token.strip(), filter( lambda token: token.strip()!='', command.split(" ")) )
    args.insert(0, "manage.py")
    
    execute_from_command_line(args)
        
    
def exists_path(*args):
    """
        Test whether a path exists.
        e.g. 
        to test /usr/lib/python use
        exists_path('usr', 'lib', 'python')
    """
    return exists(join(*args))

def exists_dir(*args):
    """
        Test whether a path exists and is a dir.
        e.g. 
        to test /usr/lib/python use
        exists_path('usr', 'lib', 'python')
    """
    if not exists_path(*args): 
        return False
    
    return isdir(join(*args))


class ColoredOutputMixin:
    """
        Mixin to print colored output,
        It should be used with classes extending BaseCommand.
    """
    def print_message(self, msg, ending='\n'):
        if ending and not msg.endswith(ending):
                msg += ending
        self.stdout.write(msg)
        
    def print_success(self, msg, ending='\n'):
        '''
            display a green message
        '''
        SUCCESS_MESSAGE = termcolors.make_style(fg='green', opts=('bold',))
        self.__display_colored_message(msg, SUCCESS_MESSAGE, ending)
    
    def print_error(self, msg, ending='\n'):
        '''
            display a red message
        '''
        ERROR_MESSAGE = termcolors.make_style(fg='red', opts=('bold',))
        self.__display_colored_message(msg, ERROR_MESSAGE, ending)
    
    def print_warning(self,msg, ending='\n'):
        '''
            display a yellow message
        '''
        WARNING_MESSAGE = termcolors.make_style(fg='yellow', opts=('bold',))
        self.__display_colored_message(msg, WARNING_MESSAGE, ending)
    
    def __display_colored_message(self, msg, style_func, ending):
        '''
            display a message given a color.
            Handle exceptions properly
        '''
        out = getattr(self, 'stdout', stdout)
        try:
            out.write(msg, style_func, ending)
        except TypeError:
            if ending and not msg.endswith(ending):
                msg += ending
            out.write(msg)

def is_blank(*args):
    if not exists_path(*args):
        return True
    
    with open(join(*args)) as f:
        body = f.read()
    
    return body.strip() == ""
        


from django.core.management.base import AppCommand
# Uncomment the next line for extending option list
# from optparse import make_option
from bricklayer.utils import exists_path, exists_dir
from django.template import loader as TemplateLoader, Context
import os
class Command(AppCommand):
    help = """Create a package for app tests.
Import all the tests defined in tests.py in the package.
To add more tests use addtest command """
    
    def handle_app(self, app, **options):
        app_label = app.__name__.split('.')[-2]
        app_path = os.path.dirname(app.__file__)

        # read tests.py file (if any)
        if exists_path(app_path, "tests.py"):
            tests_py_path = os.path.join(app_path, "tests.py")
            #Read tests.py ans save its content
            with open(tests_py_path) as f:
                tests_py = f.read()

            # Set up tests.py to work with the testsuite package
            with open(tests_py_path, 'w') as f:
                template = TemplateLoader.get_template('tests.py.template')
                context = Context({'app': app_label})
                f.write(template.render(context))

        # Create testsuite package       
        if not exists_dir(app_path, "testsuite"):
            os.mkdir(os.path.join(app_path, "testsuite"))

            
        if not exists_path(app_path, "testsuite", "__init__.py"):
            template = TemplateLoader.get_template("addtest.py.template")
            context = Context({
                "app": app_label,
                "test_name": app_label + "_tests"
                })
            with open(os.path.join(app_path, "testsuite", "__init__.py"), "w") as f:
                f.write(template.render(context))

        # Write the old tests.py in tests.APP_LABEL_tests.py
        if not exists_path(app_path, "testsuite", app_label + "_tests.py"):
            with open(os.path.join(app_path, "testsuite", app_label + "_tests.py"), "w") as f: 
                f.write(tests_py)









from django.test import TestCase
from bricklayer.utils import run_manage_py, exists_path
import test_app, os, imp

class GenerateFormTestCase(TestCase):

	def setUp(self):
		self.app_path = os.path.abspath(test_app.__path__[0])
		run_manage_py("generateform test_app -m TestModel")

	def test_file_creation(self):
		self.assertTrue(exists_path(self.app_path, "forms.py"))


	def test_form(self):
		forms = dir(imp.load_source('forms', os.path.join(self.app_path, "forms.py") ) )
		self.assertIn("TestModelForm", forms)
	

	def tearDown(self):
		os.remove(os.path.join(self.app_path, "forms.py"))



from django.core.management.base import BaseCommand, handle_default_options, OutputWrapper, OptionParser, CommandError
from django.core.management import get_commands,load_command_class
from bricklayer.management.commands import *
import sys, traceback

class Command(BaseCommand):
    help = 'Command dispatcher'
    args = "[command_name]"
    def run_from_argv(self, argv):
        """
        Command dispatcher.
        """
        
        # self.stderr is not set here
        self.stderr = OutputWrapper(sys.stderr, self.style.ERROR)
        command_name = argv.pop(2)
        
        
        command = self.__get_command_by_name(command_name)
        
        options, args = OptionParser(option_list=command.option_list).parse_args(argv[2:])
        handle_default_options(options)
        
        
        try:
            command.execute(*args, **options.__dict__)
        except Exception as e:
            if options.traceback:
                self.stderr.write(traceback.format_exc())
            else:
                self.stderr.write('%s: %s' % (e.__class__.__name__, e))
            sys.exit(1)
    

            
    def __get_command_by_name(self, command_name):
        if command_name == "admin":
            command = GenerateAdminCommand()
        elif command_name == "commandtest":
            command = TestCommand()
        elif command_name == "command":
            command = GenerateCommandCommand()
        elif command_name == "crud":
            command = GenerateCrudCommand()
        elif command_name == "form":
            command = GenerateFormCommand()
        elif command_name == "test":
            command = GenerateTestCommand()
        elif command_name == "testpackage":
            command = GenerateTestPackageCommand()
        else:
            self.stderr.write("Unknown command: %s\n" % command_name)
            sys.exit(1)
        
        return command


from django.test import TestCase
from bricklayer.utils import *
import test_app, os, imp


class GenerateTestPackageTestCase(TestCase):
	def setUp(self):
		self.app_path = os.path.abspath(test_app.__path__[0])
		with open(os.path.join(self.app_path, "tests.py")) as f:
			self.tests_content = f.read()
		run_manage_py("generatetestpackage test_app")

	def test_folder_structure(self):
		"""
			Test folder structure
		"""
		self.assertTrue(exists_dir(self.app_path, "testsuite"))
		self.assertTrue(exists_path(self.app_path, "testsuite","__init__.py"))
		self.assertTrue(exists_path(self.app_path, "testsuite","test_app_tests.py"))


	def test_testsuite_module(self):
		with open(os.path.join(self.app_path, 'testsuite', '__init__.py')) as f:
			test_text = f.read()

		self.assertIn("from test_app.testsuite.test_app_tests import *", test_text)

	def test_imports_in_tests_py(self):
		with open(os.path.join(self.app_path, 'tests.py')) as f:
			test_text = f.read()

		self.assertIn("from test_app.testsuite import *", test_text)

	def test_orginal_test_file_has_been_copied(self):
		with open(os.path.join(self.app_path, "testsuite", "test_app_tests.py")) as f:
			test_py_text = f.read()

		self.assertEqual(test_py_text.strip(), self.tests_content.strip())


	def tearDown(self):
		self.app_path = os.path.abspath(test_app.__path__[0])
		with open(os.path.join(self.app_path, "tests.py"),"w") as f:
			f.write(self.tests_content)

		for file in os.listdir(os.path.join(self.app_path, "testsuite" )):
			os.remove(os.path.join(self.app_path, "testsuite", file))

		os.rmdir(os.path.join(self.app_path, "testsuite" ))
		

		
		
		
		
		
		


from bricklayer.testsuite.generatetest import *
from bricklayer.testsuite.testsuite import *
from bricklayer.testsuite.generatecommand import *
from bricklayer.testsuite.generatecrud import *
from bricklayer.testsuite.generateadmin import *
from bricklayer.testsuite.generateform import *
from bricklayer.testsuite.generatetestpackage import *


from django.db.models import get_app
from django.core.exceptions import ImproperlyConfigured
from django.core.management.base import BaseCommand, CommandError
from django.template import loader as TemplateLoader, Context
from bricklayer.utils import exists_dir, underscore_to_camelcase, exists_path, ColoredOutputMixin
from bricklayer.management.commands.generatetestpackage import Command as TestFolderCommand
import os

class Command(BaseCommand, ColoredOutputMixin):
    help = 'Add a test file. Create a tests package if doesn\'t exist'
    args = '[test_name, app_name]'
    
    def handle(self, test_name, app_name, **options):
        try:
            app = get_app(app_name)
        except (ImproperlyConfigured, ImportError) as e:
            raise CommandError("%s. Are you sure your INSTALLED_APPS setting is correct?" % e)
        
        app_label = app.__name__.split('.')[-2]
        app_path = os.path.dirname(app.__file__)
        is_verbose = options['verbosity'] >= 1

        if not exists_dir(app_path, 'testsuite'):
            self.stdout.write("testsuite package doesn't exist. A new one is going to be created.")
            command = TestFolderCommand()
            command.handle_app(app)

        # Create a new test file named "TESTNAME.py" if it doesn't exist
        if not exists_path(app_path,'testsuite', test_name.lower()+".py" ):
            template = TemplateLoader.get_template('test.py.template')
            context = Context({
                'lower_name': test_name.lower(),
                'camel_name': underscore_to_camelcase( test_name.lower() )
                })
            with open(os.path.join(app_path, 'testsuite', test_name.lower())+".py", "w") as f:
                f.write(template.render(context))
            
            if is_verbose:
                msg = "%s.py created"%test_name
                self.print_success(msg)
                    
        # Update testsuite.__init__.py
        template = TemplateLoader.get_template('addtest.py.template')
        context = Context({
            'test_name': test_name,
            'app': app_label
            })
        
        # import the new test in the test suite
        with open(os.path.join(app_path, 'testsuite', '__init__.py'), "r+") as f:
            content = template.render(context)
            test_suite_content = f.read()
            if content not in test_suite_content:
                f.write(content)

                if is_verbose:
                    msg = "%s added to %s.testsuite"%(test_name, app_label)
                    self.print_success(msg)
            elif is_verbose:
                msg = "%s already imported in the %s.testsuite"%(test_name, app_label)
                self.print_warning(msg)








from django.test import TestCase
from bricklayer.utils import *
import test_app, os, imp


class GenerateTestTestCase(TestCase):
	def setUp(self):
		self.app_path = os.path.abspath(test_app.__path__[0])
		with open(os.path.join(self.app_path, "tests.py")) as f:
			self.tests_content = f.read()
		run_manage_py("generatetest tset test_app")

	def test_file_exists(self):
		self.assertTrue(exists_path(self.app_path, "testsuite","tset.py"))
		
	def test_init_text(self):
		with open(os.path.join(self.app_path, "testsuite", "__init__.py")) as f:
			test_file= f.read()

		self.assertIn("\nfrom test_app.testsuite.tset import *", test_file)
	
	def test_test_text(self):
		with open(os.path.join(self.app_path, "testsuite", "tset.py")) as f:
			test_file = f.read()

		self.assertIn("from django.test import TestCase", test_file)
		self.assertIn("class TsetTestCase(TestCase):", test_file)


	def tearDown(self):
		self.app_path = os.path.abspath(test_app.__path__[0])
		with open(os.path.join(self.app_path, "tests.py"),"w") as f:
			f.write(self.tests_content)
		
		for file in os.listdir(os.path.join(self.app_path, "testsuite" )):
			os.remove(os.path.join(self.app_path, "testsuite", file))

		os.rmdir(os.path.join(self.app_path, "testsuite" ))



from bricklayer.management.commands.generatetest import Command as GenerateTestCommand
from bricklayer.management.commands.generateadmin import Command as GenerateAdminCommand
from bricklayer.management.commands.generatecommand import Command as GenerateCommandCommand
from bricklayer.management.commands.generatecrud import Command as GenerateCrudCommand
from bricklayer.management.commands.generateform import Command as GenerateFormCommand
from bricklayer.management.commands.generatetestpackage import Command as GenerateTestPackageCommand
from bricklayer.management.commands.testcommand import Command as TestCommand


from django.db.models import get_models
from django.core.management.base import AppCommand, CommandError
from django.template import loader as TemplateLoader, Context
from bricklayer.utils import exists_path, ColoredOutputMixin
from optparse import make_option

import imp, os

class Command(AppCommand,ColoredOutputMixin):
    help = 'Create ModelForm for each model passed as argument.'
    # Uncomment the lines below for extending option list
    option_list = AppCommand.option_list + (
        make_option('--models', '-m', 
                    dest='models', 
                    type="string",
                    default=None,
                    help='Model/s name. Multiple models can be passed separated by a comma. E.g. -m User,Post,Comment'),
    )
    def handle_app(self, app, **options):
        
        is_verbose = options['verbosity'] >= 1
        app_label = app.__name__.split('.')[-2]
        app_path = os.path.dirname(app.__file__)
        app_models = [model._meta.object_name for model in get_models(app)]
        # Parse params
        if options['models']:
            models = options['models'].split(",")
            # Test if models exists in the app
            for model in models:
                if model not in app_models:
                    raise CommandError("No model %s in %s"%(model,app_label))
        else:
            models = app_models

        template = TemplateLoader.get_template("forms.py.template")
        if exists_path(app_path, 'forms.py'):
            # forms.py write mode   
            mode = "a"
            forms_module = dir(imp.load_source('forms', os.path.join(app_path, 'forms.py')))
            
            models_to_import = filter( lambda m: m not in forms_module, models)
            forms_to_define = filter(lambda m: m+"Form" not in forms_module, models)
            context = Context({
                "app":app_label,
                "models_to_import": models_to_import,
                "forms_to_define" : forms_to_define,
                "import_forms":     "forms" not in forms_module
                })
        else:
            mode = "w"
            context = Context({
                "app":app_label,
                "models_to_import": models,
                "forms_to_define" : models,
                "import_forms":     True
                })

        with open(os.path.join(app_path, 'forms.py'), mode) as f: 
            f.write(template.render(context))
        
        if is_verbose:
            if mode == "w":
                msg = "%s.forms created"
            else:
                msg = "%s.forms updated"
            self.print_success(msg%app_label)


from django.test import TestCase
from bricklayer.utils import run_manage_py, exists_path
import test_app, os

class GenerateAdminTestCase(TestCase):
	def setUp(self):
		self.app_path = os.path.abspath(test_app.__path__[0])
		run_manage_py("generateadmin test_app")
		
	def test_admin_py_exists(self):
		self.assertTrue(exists_path(self.app_path, 'admin.py'))

	def test_admin_py_code(self):
		with open(os.path.join(self.app_path, 'admin.py')) as f:
			source_code = "".join(f.readlines())
		#check imports
		self.assertIn("from django.contrib import admin", source_code)
		self.assertIn("from test_app.models import TestModel", source_code)
		self.assertIn("admin.site.register(TestModel, TestModelAdmin)", source_code)
		self.assertIn("class TestModelAdmin(admin.ModelAdmin):", source_code)

	def tearDown(self):
		os.remove(os.path.join(self.app_path, 'admin.py'))
		



from django.test import TestCase
from bricklayer.utils import *
import test_app
import os, imp

class GenerateCommandTestCase(TestCase):
    
    def setUp(self):
        
        self.app_path = os.path.abspath(test_app.__path__[0])
        run_manage_py("generatecommand test_command test_app")
    
    def tearDown(self):
        try:
            os.remove( os.path.join(self.app_path, "management","commands","__init__.py") )
            os.remove( os.path.join(self.app_path, "management","commands","test_command.py") )
            os.remove( os.path.join(self.app_path, "management","__init__.py") )
            os.rmdir( os.path.join(self.app_path, "management","commands") )
            os.rmdir( os.path.join(self.app_path, "management" ) )
        except: pass
    
    def test_commands(self):
        msg = ''
        # Folder creation
        self.assertTrue(exists_path( self.app_path,"management"), msg)
        self.assertTrue(os.path.isdir(os.path.join(self.app_path,"management")),msg)
        self.assertTrue(exists_path(self.app_path,"management","commands"),msg)
        self.assertTrue(os.path.isdir(os.path.join(self.app_path,"management","commands")),msg)
        # Check that are valid python packages
        self.assertTrue(exists_path(self.app_path,"management", "__init__.py"),msg)
        self.assertTrue(exists_path(self.app_path,"management","commands","__init__.py"),msg)
        self.assertTrue(exists_path(self.app_path,"management","commands","test_command.py"),msg)
    
    def test_shortcut_creation(self):
        command_module = imp.load_source('commands', os.path.join(self.app_path, 'management', 'commands', '__init__.py'))
        self.assertIn('TestCommandCommand', dir(command_module))



from django.core.management.base import BaseCommand
# Uncomment the next line for extending option list
# from optparse import make_option
from bricklayer.utils import ColoredOutputMixin

class Command(ColoredOutputMixin, BaseCommand):
    help = 'Help text goes here'
    # Uncomment the lines below for extending option list
    # option_list = BaseCommand.option_list + (
    #     make_option('--models', '-m', 
    #                 dest='models', 
    #                 type="string",
    #                 default=None,
    #                help='Model name'),
    # )
    def handle(self, *args, **options):
        self.stdout.write("normal")
        self.print_error("error")
        self.print_success("success")
        self.print_warning("warning")